@@ -6,7 +6,7 @@
 #
 # Copyright (C) 2019  Noseglasses (shinynoseglasses@gmail.com)
 #
-# This program is free software: you can redistribute it and/or modify it under it under
+# This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation, version 3.
 #
@@ -19,21 +19,25 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 #
 
-from __future__ import print_function
 import sys
+import os
+
+WARNINGS_OCCURRED: bool = False
 
 
-def eprint(*args, **kwargs):
+def printToStderr(*args, **kwargs) -> None:
+    """Print a message to stderr"""
     print(*args, file=sys.stderr, **kwargs)
 
 
-class UnrecoverableError(Exception):
-    pass
-
-
-def unrecoverableError(msg):
-    raise UnrecoverableError("{msg}".format(msg=msg))
-
-
-def warning(msg):
-    eprint("Warning: {msg}".format(msg=msg))
+def warning(msg: str) -> None:
+    """Print a warning message"""
+    warning_unicode: str
+    if os.name == "nt":
+        warning_unicode = "!!!"
+    else:
+        warning_unicode = "\u26A0"
+    warning_cascade: str = warning_unicode * 3
+    print(f"{warning_cascade} Warning: {msg} {warning_cascade}")
+    global WARNINGS_OCCURRED
+    WARNINGS_OCCURRED = True
