@@ -6,7 +6,7 @@
 #
 # Copyright (C) 2019  Noseglasses (shinynoseglasses@gmail.com)
 #
-# This program is free software: you can redistribute it and/or modify it under it under
+# This program is free software: you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
 # Foundation, version 3.
 #
@@ -18,16 +18,16 @@
 # You should have received a copy of the GNU General Public License along with along with
 # this program. If not, see <http://www.gnu.org/licenses/>.
 #
-
-from git import Repo
+from elf_diff.__init__ import __version__  # type: ignore # Make mypy ignore this module
+from elf_diff.settings import Settings
+from git import Repo  # type: ignore # Make mypy ignore this module
 import os
 
-from elf_diff.__init__ import __version__
 
+def gitRepoInfo(settings: Settings) -> str:
+    """Return the git version info. Either a SHA ID or a git tag is returned."""
 
-def gitRepoInfo(settings):
-    # If used from the git repo source tree
-    repo_path = os.path.join(settings.module_path, "..", "..")
+    repo_path: str = os.path.join(settings.module_path, "..", "..")
     if not os.path.isdir(repo_path):
         return __version__
 
