@@ -31,6 +31,7 @@
 # The nodes of the meta tree store information about what is stored, e.g. the data types
 # their documentation and means of validating the corresponding value tree nodes.
 
+import elf_diff.binary as binary
 from elf_diff.binary import Binary
 from elf_diff.binary_pair import BinaryPair, BinaryPairSettings
 from elf_diff.git import gitRepoInfo
@@ -38,7 +39,7 @@ import elf_diff.string_diff as string_diff
 from elf_diff.symbol import Symbol as ElfSymbol
 from elf_diff.binary_pair import SimilarityPair
 from elf_diff.settings import Settings
-from elf_diff.meta_tree import Node, Value, Multiple
+from elf_diff.meta_tree import Node_, Node, Value, Multiple
 from elf_diff.value_tree import Node as ValueTreeNode
 from elf_diff.meta_tree_properties import Properties, Doc, Type, AliasType
 import datetime
@@ -79,7 +80,7 @@ def _configureChildValueTreeNode(
 
 
 def _generateValueTree(
-    node: Node, parent_value_tree_node: Optional[ValueTreeNode] = None
+    node: Node_, parent_value_tree_node: Optional[ValueTreeNode] = None
 ) -> ValueTreeNode:
     """Recursively attach a value tree to the meta tree"""
     value_tree_node = ValueTreeNode()
@@ -100,13 +101,14 @@ def _generateValueTree(
     return value_tree_node
 
 
-class Symbol(Node):
+class Symbol(Node_):
     """A general symbol"""
 
     def __init__(self):
         super().__init__(
             "symbol",
             Value("name", Doc("The symbol name (demangled if supported)"), Type(str)),
+            Value("name_mangled", Doc("The mangled symbol name"), Type(str)),
             Value(
                 "is_demangled", Doc("True if the symbol name is demangled"), Type(bool)
             ),
@@ -131,6 +133,25 @@ class Symbol(Node):
                 Doc("True if the symbol is stored in program memory"),
                 Type(bool),
             ),
+            Node(
+                "source",
+                Doc("Information about the symbols source definition"),
+                Value("file_id", Doc("The id of the source file"), Type(int)),
+                Value(
+                    "line",
+                    Doc(
+                        "The line number in the source file where the symbol is defined"
+                    ),
+                    Type(int),
+                ),
+                # Value(
+                #    "column",
+                #    Doc(
+                #        "The column number in the source file where the symbol is defined"
+                #    ),
+                #    Type(int),
+                # ),
+            ),
         )
         self.connectNodes()
 
@@ -138,15 +159,19 @@ class Symbol(Node):
         """Configure the symbol meta tree node's value tree representation from an ElfSymbol"""
         symbol: ElfSymbol = kwargs["symbol"]
         value_tree_node.name = symbol.name
+        value_tree_node.name_mangled = symbol.name_mangled
         value_tree_node.is_demangled = symbol.is_demangled
         value_tree_node.type = symbol.type_
         value_tree_node.id = symbol.id_
         value_tree_node.size = symbol.size
         value_tree_node.instructions = symbol.instructions
         value_tree_node.is_stored_in_program_memory = symbol.livesInProgramMemory()
+        value_tree_node.source.file_id = symbol.source_id
+        value_tree_node.source.line = symbol.source_line
+        # value_tree_node.source.column = symbol.source_column
 
 
-class DisplayInfo(Node):
+class DisplayInfo(Node_):
     """An auxiliary display info node shared by other custom nodes"""
 
     def __init__(self):
@@ -196,7 +221,7 @@ class DisplayInfo(Node):
         value_tree_node.display_symbol_details = display_symbol_details
 
 
-class RelatedSymbols(Node):
+class RelatedSymbols(Node_):
     """Represents related symbols of a persisting symbol or a similar symbols pair"""
 
     def __init__(self):
@@ -221,7 +246,7 @@ class RelatedSymbols(Node):
         value_tree_node.size_delta = new_symbol.size - old_symbol.size
 
 
-class IsolatedSymbol(Node):
+class IsolatedSymbol(Node_):
     """An isolated symbol (either appeared or disappeared)"""
 
     def __init__(self, symbol_class: str):
@@ -265,7 +290,7 @@ class DisappearedSymbol(IsolatedSymbol):
         super().__init__(symbol_class="disappeared")
 
 
-class PersistingSymbol(Node):
+class PersistingSymbol(Node_):
     """A persisting symbol"""
 
     def __init__(self):
@@ -294,7 +319,36 @@ class PersistingSymbol(Node):
         )
 
 
-class SimilarSymbols(Node):
+class MigratedSymbol(Node_):
+    """A migrated symbol"""
+
+    def __init__(self):
+        super().__init__("migrated_symbol", DisplayInfo(), RelatedSymbols())
+        self.connectNodes()
+
+    def configureValueTree(self, value_tree_node: ValueTreeNode, **kwargs: Any) -> None:
+        """Configure the migrated symbol's associated value tree node indirectly
+        by configuring the meta tree nodes"""
+        settings: Settings = kwargs["settings"]
+        old_symbol: Symbol = kwargs["old_symbol"]
+        new_symbol: Symbol = kwargs["new_symbol"]
+        _configureChildValueTreeNode(
+            "display_info",
+            value_tree_node,
+            settings=settings,
+            symbol_class="migrated",
+            symbol1=old_symbol,
+            symbol2=new_symbol,
+        )
+        _configureChildValueTreeNode(
+            "related_symbols",
+            value_tree_node,
+            old_symbol=old_symbol,
+            new_symbol=new_symbol,
+        )
+
+
+class SimilarSymbols(Node_):
     """A similar symbols pair"""
 
     def __init__(self):
@@ -304,9 +358,9 @@ class SimilarSymbols(Node):
             RelatedSymbols(),
             Value("id", Doc("The id of the symbol pair"), Type(int)),
             Multiple(
-                ("old", "new"),
+                ["old", "new"],
                 Node(
-                    None,
+                    "",
                     Value(
                         "signature_tagged",
                         Doc(
@@ -360,10 +414,12 @@ class SimilarSymbols(Node):
 
         value_tree_node.id = id_
         value_tree_node.old.signature_tagged = string_diff.tagStringDiffSource(
-            similarity_pair.old_symbol.name, similarity_pair.new_symbol.name
+            similarity_pair.old_symbol.name,
+            similarity_pair.new_symbol.name,
         )
         value_tree_node.new.signature_tagged = string_diff.tagStringDiffTarget(
-            similarity_pair.old_symbol.name, similarity_pair.new_symbol.name
+            similarity_pair.old_symbol.name,
+            similarity_pair.new_symbol.name,
         )
         value_tree_node.similarities.signature = (
             similarity_pair.signature_similarity * 100.0
@@ -371,20 +427,54 @@ class SimilarSymbols(Node):
         value_tree_node.similarities.instruction = instruction_similarity
 
 
+class SourceFile(Node_):
+    def __init__(self):
+        super().__init__(
+            "source_file",
+            Value(
+                "path",
+                Doc("The full path of the source file as reported by binutils/nm"),
+                Type(str),
+            ),
+            Value(
+                "path_wo_prefix",
+                Doc("The name of the source file with user defined prefix stripped"),
+                Type(str),
+            ),
+            Value("id", Doc("The id of the source file"), Type(int)),
+        )
+        self.connectNodes()
+
+    def configureValueTree(self, value_tree_node: ValueTreeNode, **kwargs: Any) -> None:
+        """Configure the source file"""
+        source_file: binary.SourceFile = kwargs["source_file"]
+
+        value_tree_node.path = source_file.path
+        value_tree_node.path_wo_prefix = source_file.path_wo_prefix
+        value_tree_node.id = source_file.id_
+
+
 SYMBOL_TYPES: Tuple = (
     Symbol,
     PersistingSymbol,
     AppearedSymbol,
     DisappearedSymbol,
     SimilarSymbols,
+    MigratedSymbol,
 )
 
 SymbolType = Union[
-    Symbol, PersistingSymbol, AppearedSymbol, DisappearedSymbol, SimilarSymbols
+    SourceFile,
+    Symbol,
+    PersistingSymbol,
+    AppearedSymbol,
+    DisappearedSymbol,
+    SimilarSymbols,
+    MigratedSymbol,
 ]
 
 
-class MetaDocument(Node):
+class MetaDocument(Node_):
     """A meta document"""
 
     def __init__(self):
@@ -477,6 +567,22 @@ class MetaDocument(Node):
                     "display_similar_symbols",
                     Doc("True if similar symbols are supposed to be displayed"),
                 ),
+                Value(
+                    "display_migrated_symbols_overview",
+                    Doc(
+                        "True if an overview about migrated symbols is supposed to be displayed"
+                    ),
+                ),
+                Value(
+                    "display_migrated_symbols",
+                    Doc("True if migrated symbols are supposed to be displayed"),
+                ),
+                Value(
+                    "debug_info_available",
+                    Doc(
+                        "True if Dwarf debugging information was found in both binaries"
+                    ),
+                ),
             ),
             Value("old_binary_info", Doc("Info about the old binary"), Type(str)),
             Value("new_binary_info", Doc("Info about the new binary"), Type(str)),
@@ -488,14 +594,27 @@ class MetaDocument(Node):
                     "input",
                     Properties(Doc("Information about relevant input files")),
                     Multiple(
-                        ("old", "new"),
+                        ["old", "new"],
                         Node(
-                            None,
+                            "",
                             Value(
                                 "binary_path",
                                 Doc("The path to the binary file"),
                                 Type(str),
                             ),
+                            Value(
+                                "debug_info_available",
+                                Doc(
+                                    "True if Dwarf debug info avaiable in the elf binary"
+                                ),
+                                Type(bool),
+                            ),
+                            Value(
+                                "source_files",
+                                Doc(
+                                    "Source file by file id (dict values of type SourceFile)"
+                                ),
+                            ),
                         ),
                     ),
                 ),
@@ -506,9 +625,9 @@ class MetaDocument(Node):
                     "overall",
                     Properties(Doc("Overall statistics")),
                     Multiple(
-                        ("old", "new", "delta"),
+                        ["old", "new", "delta"],
                         Node(
-                            None,
+                            "",
                             Properties(Doc(None)),
                             Node(
                                 "resource_consumption",
@@ -529,9 +648,9 @@ class MetaDocument(Node):
                     "symbols",
                     Properties(Doc("Statistics of symbols")),
                     Multiple(
-                        ("old", "new"),
+                        ["old", "new"],
                         Node(
-                            None,
+                            "",
                             Properties(
                                 Doc("Overall statistics about symbols considered")
                             ),
@@ -563,9 +682,9 @@ class MetaDocument(Node):
                         ),
                     ),
                     Multiple(
-                        ("appeared", "disappeared"),
+                        ["appeared", "disappeared"],
                         Node(
-                            None,
+                            "",
                             Properties(Type(int), Doc(None)),
                             Value("count", Doc("Number of symbols")),
                         ),
@@ -612,6 +731,7 @@ class MetaDocument(Node):
             Node(
                 "symbols",
                 Properties(Doc("Symbols by id/table id ")),
+                Type(dict),
                 Value(
                     "old",
                     Doc(
@@ -648,6 +768,12 @@ class MetaDocument(Node):
                         "Similar symbols by symbol id (dict values of type SimilarSymbols)"
                     ),
                 ),
+                Value(
+                    "migrated",
+                    Doc(
+                        "Migrated symbols by symbol id (dict values of type MigratedSymbol)"
+                    ),
+                ),
             ),
         )
         self.connectNodes()
@@ -784,6 +910,57 @@ class MetaDocument(Node):
 
         setattr(document.symbols, "similar", value_tree_nodes)
 
+    def setupMigratedSymbolsDict(
+        self, document: ValueTreeNode, settings: Settings
+    ) -> None:
+        """Setup a dictionary of migrated symbols"""
+        value_tree_nodes: Dict[int, ValueTreeNode] = {}
+        meta_node = MigratedSymbol()
+        meta_node._name = "migrated_symbol"
+        print("Adding migrated symbols to document")
+        sys.stdout.flush()
+        for symbol_name in progressbar.progressbar(
+            self.binary_pair.migrated_symbol_names
+        ):
+            old_symbol: ElfSymbol = self.binary_pair.old_binary.symbols[symbol_name]
+            new_symbol: ElfSymbol = self.binary_pair.new_binary.symbols[symbol_name]
+
+            node = _generateValueTree(meta_node)
+            meta_node.configureValueTree(
+                node, settings=settings, old_symbol=old_symbol, new_symbol=new_symbol
+            )
+            node.related_symbols.old = document.symbols.old[old_symbol.id_]
+            node.related_symbols.new = document.symbols.new[new_symbol.id_]
+            value_tree_nodes[old_symbol.id_] = node
+
+        setattr(document.symbols, "migrated", value_tree_nodes)
+
+    @staticmethod
+    def _setupSourceFilesDict(
+        type_: str, source_files: Collection[binary.SourceFile]
+    ) -> Dict[int, ValueTreeNode]:
+        """Setup a dictionaries of source files"""
+        value_tree_nodes: Dict[int, ValueTreeNode] = {}
+        meta_node = SourceFile()
+        sys.stdout.flush()
+        for source_file in progressbar.progressbar(source_files):
+            node = _generateValueTree(meta_node)
+            meta_node.configureValueTree(node, source_file=source_file)
+            value_tree_nodes[source_file.id_] = node
+        return value_tree_nodes
+
+    def setupSourceFiles(self, document: ValueTreeNode):
+        """Setup a dictionaries of source files"""
+        old_value_tree_nodes = MetaDocument._setupSourceFilesDict(
+            "old", self.binary_pair.old_binary.source_files.values()
+        )
+        document.files.input.old.source_files = old_value_tree_nodes
+
+        new_value_tree_nodes = MetaDocument._setupSourceFilesDict(
+            "new", self.binary_pair.new_binary.source_files.values()
+        )
+        document.files.input.new.source_files = new_value_tree_nodes
+
     def configureValueTree(self, value_tree_node: ValueTreeNode, **kwargs: Any) -> None:
         """Configure the values of the document based on the information available
         from the settings
@@ -794,8 +971,8 @@ class MetaDocument(Node):
         _validateSettings(settings)
         binary_pair_settings = BinaryPairSettings(
             short_name="",
-            old_binary_filename=settings.old_binary_filename,
-            new_binary_filename=settings.new_binary_filename,
+            old_binary_filename=settings.old_binary_filename or "",
+            new_binary_filename=settings.new_binary_filename or "",
         )
         self.binary_pair = BinaryPair(
             settings=settings, pair_settings=binary_pair_settings
@@ -826,11 +1003,11 @@ class MetaDocument(Node):
             persisting_symbols_overall_size_new += new_symbol.size
             persisting_symbols_overall_size_difference += size_difference
 
-        symbol_selection_regex_old: str = old_binary.symbol_selection_regex or ".*"
-        symbol_selection_regex_new: str = new_binary.symbol_selection_regex or ".*"
+        symbol_selection_regex_old: str = settings.symbol_selection_regex_old or ".*"
+        symbol_selection_regex_new: str = settings.symbol_selection_regex_new or ".*"
 
-        symbol_exclusion_regex_old: str = old_binary.symbol_exclusion_regex or ""
-        symbol_exclusion_regex_new: str = new_binary.symbol_exclusion_regex or ""
+        symbol_exclusion_regex_old: str = settings.symbol_exclusion_regex_old or ""
+        symbol_exclusion_regex_new: str = settings.symbol_exclusion_regex_new or ""
 
         display_build_info = True
         if settings.build_info == "":
@@ -857,8 +1034,20 @@ class MetaDocument(Node):
         document.configuration.display_similar_symbols_overview = (
             not settings.skip_symbol_similarities
         )
-        document.files.input.new.binary_path = settings.new_alias
+        document.configuration.display_migrated_symbols = (
+            self.binary_pair.debug_info_available
+        )
+        document.configuration.display_migrated_symbols_overview = (
+            self.binary_pair.debug_info_available
+        )
         document.files.input.old.binary_path = settings.old_alias
+        document.files.input.new.binary_path = settings.new_alias
+        document.files.input.old.debug_info_available = (
+            self.binary_pair.old_binary.debug_info_available
+        )
+        document.files.input.new.debug_info_available = (
+            self.binary_pair.new_binary.debug_info_available
+        )
         document.general.doc_title = doc_title
         document.general.elf_diff_repo_root = settings.module_path
         document.general.generation_date = datetime.datetime.now().strftime(
@@ -870,38 +1059,51 @@ class MetaDocument(Node):
         document.new_binary_info = settings.new_binary_info
         document.old_binary_info = settings.old_binary_info
         document.statistics.overall.delta.resource_consumption.bss = (
-            new_binary.bss_size - old_binary.bss_size
+            new_binary.symbol_sizes.bss_size - old_binary.symbol_sizes.bss_size
         )
         document.statistics.overall.delta.resource_consumption.code = (
-            new_binary.progmem_size - old_binary.progmem_size
+            new_binary.symbol_sizes.progmem_size - old_binary.symbol_sizes.progmem_size
         )
         document.statistics.overall.delta.resource_consumption.data = (
-            new_binary.data_size - new_binary.data_size
+            new_binary.symbol_sizes.data_size - old_binary.symbol_sizes.data_size
         )
         document.statistics.overall.delta.resource_consumption.static_ram = (
-            new_binary.static_ram_size - old_binary.static_ram_size
+            new_binary.symbol_sizes.static_ram_size
+            - old_binary.symbol_sizes.static_ram_size
         )
         document.statistics.overall.delta.resource_consumption.text = (
-            new_binary.text_size - old_binary.text_size
+            new_binary.symbol_sizes.text_size - old_binary.symbol_sizes.text_size
+        )
+        document.statistics.overall.new.resource_consumption.bss = (
+            new_binary.symbol_sizes.bss_size
         )
-        document.statistics.overall.new.resource_consumption.bss = new_binary.bss_size
         document.statistics.overall.new.resource_consumption.code = (
-            new_binary.progmem_size
+            new_binary.symbol_sizes.progmem_size
+        )
+        document.statistics.overall.new.resource_consumption.data = (
+            new_binary.symbol_sizes.data_size
         )
-        document.statistics.overall.new.resource_consumption.data = new_binary.data_size
         document.statistics.overall.new.resource_consumption.static_ram = (
-            new_binary.static_ram_size
+            new_binary.symbol_sizes.static_ram_size
+        )
+        document.statistics.overall.new.resource_consumption.text = (
+            new_binary.symbol_sizes.text_size
+        )
+        document.statistics.overall.old.resource_consumption.bss = (
+            old_binary.symbol_sizes.bss_size
         )
-        document.statistics.overall.new.resource_consumption.text = new_binary.text_size
-        document.statistics.overall.old.resource_consumption.bss = old_binary.bss_size
         document.statistics.overall.old.resource_consumption.code = (
-            old_binary.progmem_size
+            old_binary.symbol_sizes.progmem_size
+        )
+        document.statistics.overall.old.resource_consumption.data = (
+            old_binary.symbol_sizes.data_size
         )
-        document.statistics.overall.old.resource_consumption.data = old_binary.data_size
         document.statistics.overall.old.resource_consumption.static_ram = (
-            old_binary.static_ram_size
+            old_binary.symbol_sizes.static_ram_size
+        )
+        document.statistics.overall.old.resource_consumption.text = (
+            old_binary.symbol_sizes.text_size
         )
-        document.statistics.overall.old.resource_consumption.text = old_binary.text_size
         document.statistics.symbols.appeared.count = (
             self.binary_pair.num_symbols_appeared
         )
@@ -947,6 +1149,9 @@ class MetaDocument(Node):
         self.setupDisappearedSymbolsDict(document, settings)
         self.setupPersistingSymbolsDict(document, settings)
         self.setupSimilarSymbolsDict(document, settings)
+        self.setupMigratedSymbolsDict(document, settings)
+
+        self.setupSourceFiles(document)
 
 
 def generateDocumentTree() -> ValueTreeNode:
@@ -964,11 +1169,13 @@ def generateDocument(settings: Settings) -> ValueTreeNode:
     return value_tree
 
 
-def getDocumentTreesOfSymbolClasses():
+def getDocumentTreesOfDynamicTreeNodes():
     """Returns a list that contains the meta tree nodes of all available symbol types (old/new/appeared/disappeared/persisting/similar)"""
-    tree_dumps: Dict[str, SymbolType] = {}
-    for symbol_type in SYMBOL_TYPES:
-        symbol_entity_meta: SymbolType = symbol_type()
+    tree_dumps: Dict[str, ValueTreeNode] = {}
+    node_types = [SourceFile]
+    node_types += SYMBOL_TYPES
+    for node_type in node_types:
+        symbol_entity_meta: SymbolType = node_type()
         symbol_entity_value = _generateValueTree(symbol_entity_meta)
-        tree_dumps[symbol_type.__name__] = symbol_entity_value
+        tree_dumps[node_type.__name__] = symbol_entity_value
     return tree_dumps
