@@ -20,11 +20,10 @@
 #
 
 from elf_diff.binary_pair_settings import BinaryPairSettings
-from elf_diff.error_handling import warning
+from elf_diff.binutils import Binutils
 
 import sys
 import os
-import shutil
 import argparse
 import yaml
 import datetime
@@ -42,6 +41,7 @@ class Parameter(object):
         no_cmd_line: bool = False,
         is_flag: bool = False,
         action: Optional[str] = None,
+        no_member: bool = False,
     ):
         """Initialize parameter class."""
         self.name = name
@@ -52,6 +52,7 @@ class Parameter(object):
         self.no_cmd_line = no_cmd_line
         self.is_flag = is_flag
         self.action = action
+        self.no_member = no_member
 
 
 GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
@@ -63,6 +64,21 @@ GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
             "A hint about the language that the elf was compiled from (choices: c++).",
             default="c++",
         ),
+        Parameter(
+            "source_prefix",
+            "A path prefix to remove from old and new source files (overridden by [old|new]_source_prefix)",
+            action="append",
+        ),
+        Parameter(
+            "old_source_prefix",
+            "A path prefix to remove from old source files",
+            action="append",
+        ),
+        Parameter(
+            "new_source_prefix",
+            "A path prefix to remove from new source files",
+            action="append",
+        ),
     ],
     "Report Content": [
         Parameter("project_title", "A project title to use for all reports."),
@@ -98,6 +114,12 @@ GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
             default=False,
             is_flag=True,
         ),
+        Parameter(
+            "skip_persisting_same_size",
+            "If this flag is provided, persisting symbols without size changes are skipped",
+            default=False,
+            is_flag=True,
+        ),
         Parameter(
             "consider_equal_sized_identical",
             "If this flag is defined, symbols of equal size are considered as identical (and thus ignored in most cases).",
@@ -110,11 +132,6 @@ GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
             default=False,
             is_flag=True,
         ),
-        Parameter(
-            "html_template_dir",
-            "A directory that contains template html files. Defaults to elf_diff's own html directory.",
-            no_cmd_line=True,
-        ),
     ],
     "Binutils": [
         Parameter(
@@ -128,10 +145,26 @@ GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
             default="",
         ),
         Parameter(
-            "objdump_command", "Full path to the objdump untility.", default=None
+            "objdump_command",
+            "Full path to the objdump untility.",
+            default=None,
+            no_member=True,
+        ),
+        Parameter(
+            "nm_command", "Full path to the nm untility.", default=None, no_member=True
+        ),
+        Parameter(
+            "readelf_command",
+            "Full path to the readelf untility.",
+            default=None,
+            no_member=True,
+        ),
+        Parameter(
+            "size_command",
+            "Full path to the size untility.",
+            default=None,
+            no_member=True,
         ),
-        Parameter("nm_command", "Full path to the nm untility.", default=None),
-        Parameter("size_command", "Full path to the size untility.", default=None),
     ],
     "Mangling": [
         Parameter(
@@ -154,6 +187,9 @@ GROUPED_PARAMETERS: Dict[str, List[Parameter]] = {
         Parameter("yaml_file", "The filename of the generated YAML report."),
         Parameter("json_file", "The filename of the generated JSON report."),
         Parameter("txt_file", "The filename of the generated text based report."),
+        Parameter(
+            "stats_txt_file", "The filename of the generated statistics text file."
+        ),
         Parameter("xml_file", "The filename of the generated XML report."),
         Parameter(
             "dump_document_structure",
@@ -254,10 +290,6 @@ class Settings(object):
         self.module_path: str = module_path
 
         # To enable static type checking, we have to pre-define all command line parameters
-        self.old_binary_filename: str
-        self.new_binary_filename: str
-        self.old_alias: str
-        self.new_alias: str
         self.old_info_file: str
         self.new_info_file: str
         self.build_info: str
@@ -265,6 +297,7 @@ class Settings(object):
         self.bin_prefix: str
         self.objdump_command: str
         self.nm_command: str
+        self.readelf_command: str
         self.size_command: str
         self.old_mangling_file: str
         self.new_mangling_file: str
@@ -274,16 +307,20 @@ class Settings(object):
         self.yaml_file: str
         self.json_file: str
         self.txt_file: str
+        self.stats_txt_file: str
         self.xml_file: str
         self.project_title: str
         self.driver_file: str
         self.driver_template_file: str
-        self.html_template_dir: str
         self.dump_document_structure: bool
         self.mass_report: bool
         self.language: str
+        self.source_prefix: List[str]
+        self.old_source_prefix: List[str]
+        self.new_source_prefix: List[str]
         self.similarity_threshold: float
         self.skip_symbol_similarities: bool
+        self.skip_persisting_same_size: bool
         self.consider_equal_sized_identical: bool
         self.skip_details: bool
         self.symbol_selection_regex: str
@@ -297,6 +334,8 @@ class Settings(object):
         self.list_default_plugins: bool
         self.debug: bool
 
+        self.binutils = Binutils()
+
         self._presetDefaults()
 
         cmd_line_args = Settings._parseCommandLineArgs()
@@ -307,8 +346,8 @@ class Settings(object):
         self.old_alias: Optional[str] = None
         self.new_alias: Optional[str] = None
 
-        if cmd_line_args.driver_file:
-            self.driver_file: str = cmd_line_args.driver_file
+        if hasattr(cmd_line_args, "driver_file") and cmd_line_args.driver_file:
+            self.driver_file = cmd_line_args.driver_file
             self._readDriverFile()
 
         self._considerCommandLineArgs(cmd_line_args)
@@ -320,7 +359,8 @@ class Settings(object):
         self.mass_report_members: List[BinaryPairSettings] = []
 
         for parameter in PARAMETERS:
-            setattr(self, parameter.name, parameter.default)
+            if not parameter.no_member:
+                setattr(self, parameter.name, parameter.default)
 
     @staticmethod
     def _addParameterToGroup(
@@ -411,13 +451,18 @@ class Settings(object):
                 raise Exception(exc)
 
         for parameter in PARAMETERS:
-            if parameter.name in my_yaml.keys():
+            if (parameter.name in my_yaml.keys()) and (not parameter.no_member):
                 setattr(self, parameter.name, my_yaml[parameter.name])
 
+        # Important: To make self.bin_prefix available all other parameters
+        #            must have been read from the yaml file already.
+        self.binutils.initialize(
+            my_yaml, bin_prefix=self.bin_prefix, bin_dir=self.bin_dir
+        )
+
         # Read binary pairs
 
         if "binary_pairs" in my_yaml.keys():
-
             bin_pair_id: int = 1
 
             for data_set in my_yaml["binary_pairs"]:
@@ -455,9 +500,15 @@ class Settings(object):
             if hasattr(cmd_line_args, parameter.name):
                 value = getattr(cmd_line_args, parameter.name)
 
-                if value != parameter.default:
+                if (value != parameter.default) and (not parameter.no_member):
                     setattr(self, parameter.name, value)
 
+        # Important: To make self.bin_prefix available the command line arguments for
+        #            all other parameters must have been read from the yaml file already.
+        self.binutils.initialize(
+            cmd_line_args.__dict__, bin_prefix=self.bin_prefix, bin_dir=self.bin_dir
+        )
+
         if len(cmd_line_args.binaries) == 0:
             pass
         elif len(cmd_line_args.binaries) == 2:
@@ -473,61 +524,6 @@ class Settings(object):
         else:
             raise Exception("Please specify either none or two binaries")
 
-    def _findUtilityInBinDir(
-        self, name: str, exe_extensions: List[str]
-    ) -> Optional[str]:
-        for exe_extension in exe_extensions:
-            basename: str = self.bin_prefix + name + exe_extension
-
-            if self.bin_dir is not None:
-                command = os.path.join(self.bin_dir, basename)
-                if (os.path.isfile(command)) and (os.access(command, os.X_OK)):
-                    return command
-        return None
-
-    def _findUtilityUsingWhich(
-        self, name: str, exe_extensions: List[str]
-    ) -> Optional[str]:
-        for exe_extension in exe_extensions:
-            basename = self.bin_prefix + name + exe_extension
-            command = shutil.which(basename)
-            if (
-                (command is not None)
-                and (os.path.isfile(command))
-                and (os.access(command, os.X_OK))
-            ):
-                return command
-        return None
-
-    def findUtility(self, name: str) -> None:
-        """Find a utility and set a attribute of this class to the path of the utility executable"""
-        command_name: str = name + "_command"
-        command: Optional[str] = getattr(self, command_name)
-        if command is not None:
-            if (os.path.isfile(command)) and (os.access(command, os.X_OK)):
-                return
-            warning(f"Unable to find predefined {command_name} = {command}")
-
-        exe_extensions: List[str]
-        if os.name == "nt":
-            exe_extensions = [".exe", ""]
-        else:
-            exe_extensions = ["", ".exe"]
-
-        command = self._findUtilityInBinDir(name, exe_extensions)
-
-        if command is not None:
-            setattr(self, command_name, command)
-            return
-
-        command = self._findUtilityUsingWhich(name, exe_extensions)
-
-        if command is not None:
-            setattr(self, command_name, command)
-            return
-
-        raise Exception(f"Unnable to find {name} command")
-
     def _validateBinaries(self) -> None:
         """Validate and initialize the settings"""
         if self.old_binary_filename and not os.path.isfile(self.old_binary_filename):
@@ -575,34 +571,35 @@ class Settings(object):
     def _validateAndInitSettings(self) -> None:
         self._validateBinaries()
 
-        self.findUtility("objdump")
-        self.findUtility("nm")
-        self.findUtility("size")
-
-        print("Tools:")
-        print(f"   objdump: {self.objdump_command}")
-        print(f"   nm:      {self.nm_command}")
-        print(f"   size:    {self.size_command}")
-
         self._prepareInfoFiles()
         self._prepareAlias()
 
+    @staticmethod
+    def _writeTemplateFileValue(file_, name: str, value: Any):
+        file_.write("# {desc}\n".format(desc=parameter.description))
+        file_.write("#\n")
+        file_.write('{name}: "{value}"\n'.format(name=parameter.name, value=value))
+        file_.write("\n")
+
     def writeParameterTemplateFile(
         self, filename: str, output_actual_values: bool = False
     ) -> None:
         """Write a template file with all existing parameters"""
-        with open(filename, "w") as f:
+        with open(filename, "w") as file_:
 
-            f.write("# This is an auto generated elf_diff driver file\n")
-            f.write(
+            file_.write("# This is an auto generated elf_diff driver file\n")
+            file_.write(
                 "# Generated by elf_diff {date}\n".format(
                     date=datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")
                 )
             )
-            f.write("\n")
+            file_.write("\n")
 
             for parameter in PARAMETERS:
 
+                if parameter.no_member:
+                    continue
+
                 if output_actual_values:
                     value = getattr(self, parameter.name)
                     if not value:
@@ -611,10 +608,15 @@ class Settings(object):
                 else:
                     value = parameter.default
 
-                f.write("# {desc}\n".format(desc=parameter.description))
-                f.write("#\n")
-                f.write('{name}: "{value}"\n'.format(name=parameter.name, value=value))
-                f.write("\n")
+                Settings._writeTemplateFileValue(
+                    file_=file_, name=parameter.name, value=value
+                )
+
+            for command in Binutils.COMMANDS:
+                name = "%s_command" % command
+                value = getattr(self.binutils, name)
+
+                Settings._writeTemplateFileValue(file_=file_, name=name, value=value)
 
     def isFirmwareBinaryDefined(self) -> bool:
         """Check if any firmware binary is defined"""
