@@ -19,16 +19,37 @@
 # this program. If not, see <http://www.gnu.org/licenses/>.
 #
 
-from elf_diff.plugin import ExportPairReportPlugin
+from elf_diff.plugin import (
+    ExportPairReportPlugin,
+    PluginConfigurationInformation,
+    PluginConfigurationKey,
+)
+from elf_diff.settings import Settings
+from elf_diff.pair_report_document import ValueTreeNode
+from typing import Dict
 
 
 class TestExportPairReportPlugin(ExportPairReportPlugin):
-    def __init__(self, settings, plugin_configuration):
+    def __init__(self, settings: Settings, plugin_configuration: Dict[str, str]):
         super().__init__(settings, plugin_configuration)
 
     def export(
-        self, document
+        self, document: ValueTreeNode
     ):  # pylint: disable=arguments-differ # There's no visible reason why pylint warns here
         print("Test plugin configuration:")
-        for key, value in self.plugin_configuration.items():
+        for key, value in self._plugin_configuration.items():
             print("   %s = %s" % (key, value))
+
+    @staticmethod
+    def getConfigurationInformation() -> PluginConfigurationInformation:
+        """Return plugin configuration information"""
+        return [
+            PluginConfigurationKey(
+                "magic_words",
+                "Important words",
+                is_optional=True,
+                default="Hocus pocus",
+            )
+        ] + super(
+            TestExportPairReportPlugin, TestExportPairReportPlugin
+        ).getConfigurationInformation()
